@@ -1,4 +1,4 @@
-import re
+import json
 import subprocess
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 from rich.progress import Progress, TaskID, Task
@@ -43,7 +43,9 @@ def run_cmd(
     )
 
 
-def shorten_filepath(in_path: str, max_length: int) -> str:
+def shorten_filepath(in_path: Union[str, Path], max_length: int) -> str:
+    in_path = str(in_path)
+
     if len(in_path) > max_length:
         if ":" in in_path:
             in_path = (
@@ -56,31 +58,6 @@ def shorten_filepath(in_path: str, max_length: int) -> str:
         return in_path
 
 
-def convert2bits(value: float, unit: str) -> float:
-    """Returns the corresponding bit value to a value with a certain binary prefix (based on powers of 2) like KiB or MiB.
-
-    Args:
-        value (float): Bit value using a certain binary prefix like KiB or MiB.
-        unit (str): The binary prefix.
-
-    Returns:
-        float: The corresponding bit value.
-    """
-    exp = {
-        "B": 0,
-        "KiB": 1,
-        "MiB": 2,
-        "GiB": 3,
-        "TiB": 4,
-        "PiB": 5,
-        "EiB": 6,
-        "ZiB": 7,
-        "YiB": 8,
-    }
-
-    return value * 1024 ** exp[unit]
-
-
 # ---------------------------------------------------------------------------- #
 #                          Progressbar related functions                       #
 # ---------------------------------------------------------------------------- #
@@ -89,13 +66,11 @@ def convert2bits(value: float, unit: str) -> float:
 def rclone_progress(
     command: str,
     pbar_title: str,
-    stderr=subprocess.PIPE,
     show_progress=True,
     listener: Callable[[Dict], None] = None,
     debug=False,
     pbar: Optional[Progress] = None,
 ) -> subprocess.Popen:
-    buffer = ""
     total_progress_id = None
     subprocesses = {}
 
@@ -106,12 +81,14 @@ def rclone_progress(
         total_progress_id = pbar.add_task(pbar_title, total=None)
 
     process = subprocess.Popen(
-        command, stdout=subprocess.PIPE, stderr=stderr, shell=True
+        args=command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True
     )
-    for line in iter(process.stdout.readline, b""):
-        var = line.decode()
 
-        valid, update_dict = extract_rclone_progress(buffer)
+    # rclone prints stats to stderr. each line is one update
+    for line in iter(process.stderr.readline, b""):
+        line = line.decode()
+
+        valid, update_dict = extract_rclone_progress(line)
 
         if valid:
             if show_progress:
@@ -122,13 +99,7 @@ def rclone_progress(
                 listener(update_dict)
 
             if debug:
-                pbar.log(buffer)
-
-            # reset the buffer
-            buffer = ""
-        else:
-            # buffer until we
-            buffer += var
+                pbar.log(line)
 
     if show_progress:
         complete_task(total_progress_id, pbar)
@@ -140,48 +111,52 @@ def rclone_progress(
     return process
 
 
-def extract_rclone_progress(buffer: str) -> Tuple[bool, Union[Dict[str, Any], None]]:
-    # matcher that checks if the progress update block is completely buffered yet (defines start and stop)
-    # it gets the sent bits, total bits, progress, transfer-speed and eta
-    reg_transferred = re.findall(
-        r"Transferred:\s+(\d+.\d+ \w+) \/ (\d+.\d+ \w+), (\d{1,3})%, (\d+.\d+ \w+\/\w+), ETA (\S+)",
-        buffer,
-    )
+def extract_rclone_progress(line: str) -> Tuple[bool, Union[Dict[str, Any], None]]:
+    """Extracts and returns the progress updates from the rclone transfer operation.
+    The returned Dictionary includes the original rclone stats output inside of "rclone_output".
+    All file sizes and speeds are give in bytes.
 
-    if reg_transferred:  # transferred block is completely buffered
+    Args:
+        line (str): One output line of the rclone transfer operation with the --use-json-log flag enabled.
+
+    Returns:
+        Tuple[bool, Union[Dict[str, Any], None]]: The retrieved update Dictionary.
+    """
+
+    try:
+        stats: Dict = json.loads(line).get("stats", None)
+    except ValueError:
+        stats = None
+
+    if stats is not None and stats.get("totalBytes", 0) > 0:
         # get the progress of the individual files
-        # matcher gets the currently transferring files and their individual progress
-        # returns list of tuples: (name, progress, file_size, unit)
-        prog_transferring = []
-        prog_regex = re.findall(
-            r"\* +(.+):[ ]+(\d{1,3})% \/(\d+.\d+)([a-zA-Z]+),", buffer
-        )
-        for item in prog_regex:
-            prog_transferring.append(
-                (
-                    item[0],
-                    int(item[1]),
-                    float(item[2]),
-                    # the suffix B of the unit is missing for subprocesses
-                    item[3] + "B",
-                )
+        tasks = []
+        for t in stats.get("transferring", []):
+            total = t.get("size", 0)
+            sent = t.get("bytes", 0)
+            tasks.append(
+                {
+                    # sometime not all the task information is available right from the start
+                    "name": t.get("name", "N/A"),
+                    "total": total,
+                    "sent": sent,
+                    "progress": sent / total if total != 0 else 0,
+                    "transfer_speed": t.get("speed", 0),
+                }
             )
 
-        out = {"prog_transferring": prog_transferring}
-        sent_bits, total_bits, progress, transfer_speed_str, eta = reg_transferred[0]
-        out["progress"] = float(progress.strip())
-        out["total_bits"] = float(re.findall(r"\d+.\d+", total_bits)[0])
-        out["sent_bits"] = float(re.findall(r"\d+.\d+", sent_bits)[0])
-        out["unit_sent"] = re.findall(r"[a-zA-Z]+", sent_bits)[0]
-        out["unit_total"] = re.findall(r"[a-zA-Z]+", total_bits)[0]
-        out["transfer_speed"] = float(re.findall(r"\d+.\d+", transfer_speed_str)[0])
-        out["transfer_speed_unit"] = re.findall(
-            r"[a-zA-Z]+/[a-zA-Z]+", transfer_speed_str
-        )[0]
-        out["eta"] = eta
+        out = {
+            "tasks": tasks,
+            "total": stats["totalBytes"],
+            "sent": stats["bytes"],
+            "progress": (
+                stats["bytes"] / stats["totalBytes"] if stats["totalBytes"] != 0 else 0
+            ),
+            "transfer_speed": stats["speed"],
+            "rclone_output": stats,
+        }
 
         return True, out
-
     else:
         return False, None
 
@@ -251,35 +226,40 @@ def update_tasks(
 
     pbar.update(
         total_progress,
-        completed=convert2bits(update_dict["sent_bits"], update_dict["unit_sent"]),
-        total=convert2bits(update_dict["total_bits"], update_dict["unit_total"]),
+        completed=update_dict["sent"],
+        total=update_dict["total"],
     )
 
-    sp_names = set()
-    for sp_file_name, sp_progress, sp_size, sp_unit in update_dict["prog_transferring"]:
+    task_names = set()
+    for task in update_dict["tasks"]:
         task_id = None
-        sp_names.add(sp_file_name)
 
-        if sp_file_name not in subprocesses:
+        task_name = task["name"]
+        task_size = task["total"]
+        task_sent = task["sent"]
+
+        task_names.add(task_name)
+
+        if task_name not in subprocesses:
             task_id = pbar.add_task(" ", visible=False)
-            subprocesses[sp_file_name] = task_id
+            subprocesses[task_name] = task_id
         else:
-            task_id = subprocesses[sp_file_name]
+            task_id = subprocesses[task_name]
 
         pbar.update(
             task_id,
             # set the description every time to reset the '├'
-            description=f" ├─{sp_file_name}",
-            completed=convert2bits(sp_size, sp_unit) * sp_progress / 100.0,
-            total=convert2bits(sp_size, sp_unit),
+            description=f" ├─{task_name}",
+            completed=task_sent,
+            total=task_size,
             # hide subprocesses if we only upload a single file
             visible=len(subprocesses) > 1,
         )
 
     # make all processes invisible that are no longer provided by rclone (bc. their upload completed)
-    missing = list(sorted(subprocesses.keys() - sp_names))
-    for missing_sp_id in missing:
-        pbar.update(subprocesses[missing_sp_id], visible=False)
+    missing = list(sorted(subprocesses.keys() - task_names))
+    for missing_task_id in missing:
+        pbar.update(subprocesses[missing_task_id], visible=False)
 
     # change symbol for the last visible process
     for task in reversed(pbar.tasks):
